package ru.dbotthepony.kstarbound.defs

import com.google.common.collect.ImmutableMap
import it.unimi.dsi.fastutil.objects.Object2ObjectArrayMap

/**
 * Базовый класс описания прототипа игрового объекта
 *
 * Должен иметь все (или больше) поля объекта, который он будет создавать
 *
 * Поля должны иметь базовые ограничения (т.е. ограничения, которые применимы для всех конфигураций прототипа).
 * Если границы поля зависят от других полей, то проверка такого поля должна осуществляться уже при самой
 * сборке прототипа.
 */
abstract class RawPrototype<RAW : RawPrototype<RAW, ASSEMBLED>, ASSEMBLED : AssembledPrototype<ASSEMBLED, RAW>> {
	val json = Object2ObjectArrayMap<String, Any>()
	fun enroll() = enrollMap(json)
	abstract fun assemble(directory: String = ""): ASSEMBLED
}

/**
 * Базовый класс описанного прототипа игрового объекта
 *
 * Должен иметь все поля объекта, которые будут использоваться движком напрямую
 *
 * Создается соответствующим [RawPrototype], который проверил уже все поля
 * на их правильность.
 */
abstract class AssembledPrototype<ASSEMBLED : AssembledPrototype<ASSEMBLED, RAW>, RAW : RawPrototype<RAW, ASSEMBLED>>(
	val json: ImmutableMap<String, Any>
) {
	open fun getParameter(key: String): Any? = json[key]
	fun unroll() = flattenMap(json)
	abstract fun disassemble(): RAW
}
